package com.changhong.epc.form.filter;

import com.changhong.autoform.core.filter.AutoFormProcessStateFilter;
import com.changhong.autoform.entity.WorkFlowStateEntity;
import com.changhong.autoform.entity.system.RestResult;
import com.changhong.autoform.entity.system.RestResultGenerator;
import com.changhong.epc.bean.tenant.system.SystemConfig;
import com.changhong.epc.constter.base.BaseConst;
import com.changhong.epc.constter.form.FormConst;
import com.changhong.epc.constter.system.SystemClients;
import com.changhong.epc.constter.system.businesscode.BusinessMeaningCode;
import com.changhong.epc.constter.tenant.TenantUrlConst;
import com.changhong.epc.form.service.budget.tool.IUpdateBudget;
import com.changhong.epc.form.service.budget.tool.UpdateBudgetFactory;
import com.changhong.epc.form.service.budget.tool.entity.UpdateBudgetEntity;
import com.changhong.epc.rely.api.epc.tenant.SystemConfigApi;
import com.iemsoft.framework.cloud.core.exception.IEMRuntimeException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Objects;

@Slf4j
@Service
public class AutoFormProStatesFilter implements SystemClients, TenantUrlConst, AutoFormProcessStateFilter , BaseConst, BusinessMeaningCode, FormConst {

	@Resource
	private UpdateBudgetFactory updateBudgetFactory;

	@Resource
	private SystemConfigApi systemConfigApi;

	/**
	 * 流程执行前
	 */
	@Override
	public RestResult<Boolean> filterWorkFlowBefor(WorkFlowStateEntity workFlowStateEntity) {
		try {
			UpdateBudgetEntity updateBudgetEntity = new UpdateBudgetEntity();
			updateBudgetEntity.setFormId(workFlowStateEntity.getFormId());
			updateBudgetEntity.setDataRowNum(workFlowStateEntity.getDataRowNum());
			IUpdateBudget updateBudget = updateBudgetFactory.getIUpdateBudget(updateBudgetEntity);
			if (updateBudget == null) {
				return RestResultGenerator.ok(Boolean.TRUE);
			}
			updateBudget.checkBudget(updateBudgetEntity);
			return RestResultGenerator.ok(Boolean.TRUE);
		}catch (IEMRuntimeException e){
			log.error(e.getMessage(), e);
			RestResult<Boolean> result = RestResult.newInstance();
			result.setRspCode(e.getCode());
			result.setRspDesc(e.getMessage());
			return result;
		}catch (Exception e){
			log.error(e.getMessage(), e);
			throw e;
		}
	}

	/**
	 * 流程执行后
	 * @param workFlowStateEntity
	 * @return
	 */
	@Override
	public RestResult<Boolean> filterWorkFlowAfter(WorkFlowStateEntity workFlowStateEntity) {
		try {
			UpdateBudgetEntity updateBudgetEntity = new UpdateBudgetEntity();
			updateBudgetEntity.setFormId(workFlowStateEntity.getFormId());
			updateBudgetEntity.setDataRowNum(workFlowStateEntity.getDataRowNum());
			IUpdateBudget updateBudget = updateBudgetFactory.getIUpdateBudget(updateBudgetEntity);
			if (updateBudget == null) {
				return RestResultGenerator.ok(Boolean.TRUE);
			}
			return RestResultGenerator.ok(Boolean.TRUE);
		}catch (IEMRuntimeException e){
			log.error(e.getMessage(), e);
			RestResult<Boolean> result = RestResult.newInstance();
			result.setRspCode(e.getCode());
			result.setRspDesc(e.getMessage());
			return result;
		}catch (Exception e){
			log.error(e.getMessage(), e);
			throw e;
		}
	}

	@Override
	public RestResult<Boolean> filterWorkFlowCreate(WorkFlowStateEntity workFlowStateEntity) {
		try {
			UpdateBudgetEntity updateBudgetEntity = new UpdateBudgetEntity();
			updateBudgetEntity.setFormId(workFlowStateEntity.getFormId());
			updateBudgetEntity.setDataRowNum(workFlowStateEntity.getDataRowNum());
			IUpdateBudget updateBudget = updateBudgetFactory.getIUpdateBudget(updateBudgetEntity);
			if (updateBudget == null) {
				return RestResultGenerator.ok(Boolean.TRUE);
			}
			if(updateBudgetFactory.isAutoBudgetUpdate(updateBudget)) {
				if (Objects.equals("1", systemConfigApi.getVal(SystemConfig.ofKey("budget", "deductions"), SystemConfig::getCvalue))) {
					updateBudget.updateBudget(updateBudgetEntity);
				}
			}

			return RestResultGenerator.ok(Boolean.TRUE);
		}catch (IEMRuntimeException e){
			log.error(e.getMessage(), e);
			RestResult<Boolean> result = RestResult.newInstance();
			result.setRspCode(e.getCode());
			result.setRspDesc(e.getMessage());
			return result;
		}catch (Exception e){
			log.error(e.getMessage(), e);
			throw e;
		}
	}

	@Override
	public RestResult<Boolean> filterWorkFlowRollback(WorkFlowStateEntity workFlowStateEntity) {
		try {
			UpdateBudgetEntity updateBudgetEntity = new UpdateBudgetEntity();
			updateBudgetEntity.setFormId(workFlowStateEntity.getFormId());
			updateBudgetEntity.setDataRowNum(workFlowStateEntity.getDataRowNum());
			IUpdateBudget updateBudget = updateBudgetFactory.getIUpdateBudget(updateBudgetEntity);
			if (updateBudget == null) {
				return RestResultGenerator.ok(Boolean.TRUE);
			}
			if(updateBudgetFactory.isAutoBudgetUpdate(updateBudget)) {
				if (Objects.equals("1", systemConfigApi.getVal(SystemConfig.ofKey("budget", "deductions"), SystemConfig::getCvalue))) {
					updateBudget.rollback(updateBudgetEntity);
				}
			}

			return RestResultGenerator.ok(Boolean.TRUE);
		}catch (IEMRuntimeException e){
			log.error(e.getMessage(), e);
			RestResult<Boolean> result = RestResult.newInstance();
			result.setRspCode(e.getCode());
			result.setRspDesc(e.getMessage());
			return result;
		}catch (Exception e){
			log.error(e.getMessage(), e);
			throw e;
		}
	}

	/**
	 * 流程结束后
	 * @param workFlowStateEntity
	 * @return
	 */
	@Override
	public RestResult<Boolean> filterWorkFlowOver(WorkFlowStateEntity workFlowStateEntity) {
		try {
			UpdateBudgetEntity updateBudgetEntity = new UpdateBudgetEntity();
			updateBudgetEntity.setFormId(workFlowStateEntity.getFormId());
			updateBudgetEntity.setDataRowNum(workFlowStateEntity.getDataRowNum());
			IUpdateBudget updateBudget = updateBudgetFactory.getIUpdateBudget(updateBudgetEntity);
			if (updateBudget == null) {
				return RestResultGenerator.ok(Boolean.TRUE);
			}
			if(updateBudgetFactory.isAutoBudgetUpdate(updateBudget)) {
				if (!Objects.equals("1", systemConfigApi.getVal(SystemConfig.ofKey("budget", "deductions"), SystemConfig::getCvalue))) {
					updateBudget.updateBudget(updateBudgetEntity);
				}
			}else{
				updateBudget.updateBudget(updateBudgetEntity);
			}
			updateBudget.flowAfterCall(updateBudgetEntity);

			return RestResultGenerator.ok(Boolean.TRUE);
		}catch (IEMRuntimeException e){
			log.error(e.getMessage(), e);
			RestResult<Boolean> result = RestResult.newInstance();
			result.setRspCode(e.getCode());
			result.setRspDesc(e.getMessage());
			return result;
		}catch (Exception e){
			log.error(e.getMessage(), e);
			throw e;
		}
	}
}