package com.changhong.epc.form.filter.data.impl;

import com.changhong.epc.form.filter.data.FormDataFilter;
import com.iemsoft.framework.cloud.core.tools.JSONTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * 从固化单据中提取下拉框的name
 */
@Service
@Slf4j
public class FormDataCodeToNameFilter implements FormDataFilter {

    @Override
    public void accept(Map<String, Object> data) {
        accept(data, data, 0);
    }

    public void accept(Map<String, Object> data, List<Map<String, Object>> subs){
        int i = 0;
        for (Map<String, Object> sub : subs){
            int index = Integer.valueOf(Objects.toString(sub.get("index"), "-1"));
            accept(data, sub, index>0?index:i);
            i++;
        }
    }

    public void accept(Map<String, Object> data, Map<String, Object> subs){
        try {
            String solidification = Objects.toString(data.get("solidification"));
            if (ObjectUtil.empty(solidification)) {
                return;
            }
            Map<String, Map<String, Object>> solidificationMap = toMap(JSONTool.toList(solidification, Map.class));
            subs.entrySet()
                    .stream()
                    .forEach(entity -> {
                        Map<String, Object> res = solidificationMap.get(entity.getKey());
                        if (res == null || res.get("value") == null || res.get("name") == null) {
                            return;
                        }
                        if(((List) res.get("value")).size() != ((List) res.get("name")).size()){
                            return;
                        }
                        int i = 0;
                        boolean flag = false;
                        for (Object val : ((List) res.get("value"))) {
                            if (Objects.equals(val, entity.getValue())) {
                                flag = true;
                                break;
                            }
                            i++;
                        }
                        if(flag && ObjectUtil.notEmpty(res.get("name")))
                            entity.setValue(((List) res.get("name")).get(i));
                    });
        }catch (Exception e){
            log.error(e.getMessage(), e);
        }
    }

    protected void accept(Map<String, Object> data, Map<String, Object> info, int index){
        log.debug("主表单:{}, 子表单:{}, 下标:{}", JSONTool.toJson(data), JSONTool.toJson(info), index);
        String solidification = Objects.toString(data.get("solidification"));
        if(ObjectUtil.empty(solidification)){
            return;
        }
        Map<String, Map<String, Object>> solidificationMap = toMap(JSONTool.toList(solidification, Map.class));
        info.entrySet()
                .stream()
                .forEach(entity-> {
                    String val = Optional.ofNullable(solidificationMap.get(entity.getKey()))
                            .map(map->map.get("name"))
                            .filter(map->map instanceof List)
                            .filter(map->((List)map).size()>0)
                            .map(list->((List) list).get(index))
                            .map(Object::toString)
                            .orElse("");
                    if(ObjectUtil.empty(val)){
                        return;
                    }
                    entity.setValue(val);
                });
    }

    Map<String, Map<String, Object>> toMap(List<Map> solidificationObj){
        if(ObjectUtil.empty(solidificationObj)){
            return Collections.EMPTY_MAP;
        }
        return solidificationObj
                .stream()
                .reduce(new HashMap(), (m1, m2)->{
                    m1.put(m2.get("alias"), m2);
                    return m1;
                });
    }

//    public static void main(String[] args) {
////        System.out.println("{\"DELETEFLG\":\"0\",\"budgetStatement\":\"下级预算\",\"occupy\":\"F000002\",\"processState\":\"30\",\"processTask\":\" \",\"TENANTID\":\"100000001\",\"CREATEDATETIME\":1558684933000,\"solidification\":\"[{\\\"alias\\\":\\\"budget_type\\\",\\\"tag_Id\\\":\\\"itemId_2\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"F000001\\\"],\\\"name\\\":[\\\"部门预算\\\"]},{\\\"alias\\\":\\\"currency\\\",\\\"tag_Id\\\":\\\"itemId_3\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"RMB\\\"],\\\"name\\\":[\\\"人民币\\\"]},{\\\"alias\\\":\\\"occupy\\\",\\\"tag_Id\\\":\\\"itemId_4\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"F000002\\\"],\\\"name\\\":[\\\"使用\\\"]},{\\\"alias\\\":\\\"budgetStatement\\\",\\\"tag_Id\\\":\\\"itemId_7\\\",\\\"tag_Type\\\":\\\"text\\\",\\\"value\\\":\\\"下级预算\\\"},{\\\"alias\\\":\\\"C_Type\\\",\\\"tag_Id\\\":\\\"itemId_11\\\",\\\"tag_Type\\\":\\\"tree\\\",\\\"value\\\":[\\\"51\\\",\\\"371\\\",\\\"151\\\",\\\"695\\\"],\\\"name\\\":[\\\"办公费\\\",\\\"差旅费\\\",\\\"通讯费\\\",\\\"折旧费\\\"]},{\\\"alias\\\":\\\"budget_itemDepartment\\\",\\\"tag_Id\\\":\\\"itemId_12\\\",\\\"tag_Type\\\":\\\"tree\\\",\\\"value\\\":[\\\"01010000100005000001\\\",\\\"01010000100005000001\\\",\\\"01010000100005000001\\\",\\\"01010000100005000001\\\"],\\\"name\\\":[\\\"运营管理处\\\",\\\"运营管理处\\\",\\\"运营管理处\\\",\\\"运营管理处\\\"]},{\\\"alias\\\":\\\"budget_superDepartement\\\",\\\"tag_Id\\\":\\\"itemId_17\\\",\\\"tag_Type\\\":\\\"tree\\\",\\\"value\\\":[\\\"010100001\\\",\\\"010100001\\\",\\\"010100001\\\",\\\"010100001\\\"],\\\"name\\\":[\\\"四川长虹模塑科技有限公司合肥分公司\\\",\\\"四川长虹模塑科技有限公司合肥分公司\\\",\\\"四川长虹模塑科技有限公司合肥分公司\\\",\\\"四川长虹模塑科技有限公司合肥分公司\\\"]},{\\\"alias\\\":\\\"budget_project\\\",\\\"tag_Id\\\":\\\"itemId_18\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[],\\\"name\\\":[]}]\",\"budget_subFrom\":[{\"DELETEFLG\":\"0\",\"budget_superDepartement\":\"010100001\",\"C_Type\":\"371\",\"budget_itemMoney\":\"6000.00\",\"budget_itemDepartment\":\"01010000100005000001\",\"processState\":\"10\",\"TENANTID\":\"100000001\",\"CREATEDATETIME\":1558684933000,\"orderStatus\":\"20\",\"budgetEndDate\":\"2019-05-25\",\"budgetStartDate\":\"2019-05-24\",\"number_budget\":\"B00000065\",\"budget_itemBalance\":\"6000.00\",\"budget_project\":\"\",\"UPDATEDATETIME\":1558684933000,\"budget_superMoney\":\"5000.00\",\"PARENTDATAROWNUM\":1127,\"DATAROWNUM\":2684}],\"toWriteBudget\":\"1125\",\"processUser\":\"3141bf6b126f4f16\",\"htmlUrl\":\"group1/M00/01/AB/CgQgJVzno8-AMDysAADhCfnPSLY06.json\",\"orderStatus\":\"40\",\"budgetEndDate\":\"2019-05-25\",\"budgetStartDate\":\"2019-05-24\",\"number_budget\":\"B00000065\",\"CREATEUSER\":\"3141bf6b126f4f16\",\"UPDATEDATETIME\":1558684933000,\"currency\":\"RMB\",\"PARENTDATAROWNUM\":0,\"budget_type\":\"F000001\",\"DATAROWNUM\":1127}");
//        Map<String, Object> data = JSONTool.toObj("{\"occupy\":\"YSCZ000002\",\"budgetStatement\":\"财务处公共预算\",\"processState\":\"30\",\"processStatE\":\"30\",\"solidification\":\"[{\\\"alias\\\":\\\"budget_type\\\",\\\"tag_Id\\\":\\\"itemId_2\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"F090002\\\"],\\\"name\\\":[\\\"项目预算\\\"]},{\\\"alias\\\":\\\"currency\\\",\\\"tag_Id\\\":\\\"itemId_3\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"RMB\\\"],\\\"name\\\":[\\\"人民币\\\"]},{\\\"alias\\\":\\\"occupy\\\",\\\"tag_Id\\\":\\\"itemId_4\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"YSCZ000002\\\"],\\\"name\\\":[\\\"使用\\\"]},{\\\"alias\\\":\\\"budgetStatement\\\",\\\"tag_Id\\\":\\\"itemId_7\\\",\\\"tag_Type\\\":\\\"text\\\",\\\"value\\\":\\\"财务处公共预算\\\"},{\\\"alias\\\":\\\"C_Type\\\",\\\"tag_Id\\\":\\\"itemId_11\\\",\\\"tag_Type\\\":\\\"tree\\\",\\\"value\\\":[\\\"165\\\",\\\"51\\\",\\\"509\\\",\\\"149\\\",\\\"371\\\",\\\"557\\\",\\\"1344\\\",\\\"1345\\\"],\\\"name\\\":[\\\"业务活动费\\\",\\\"办公费\\\",\\\"信息咨询费\\\",\\\"培训费\\\",\\\"差旅费\\\",\\\"审计评估费\\\",\\\"服务支持费\\\",\\\"财产保险费\\\"]},{\\\"alias\\\":\\\"budget_itemDepartment\\\",\\\"tag_Id\\\":\\\"itemId_12\\\",\\\"tag_Type\\\":\\\"tree\\\",\\\"value\\\":[\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\",\\\"01437000100004\\\"],\\\"name\\\":[\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\",\\\"财务处\\\"]},{\\\"alias\\\":\\\"budget_project\\\",\\\"tag_Id\\\":\\\"itemId_18\\\",\\\"tag_Type\\\":\\\"dropdownlist\\\",\\\"value\\\":[\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\",\\\"SDT-CWC-GGXM-2019\\\"],\\\"name\\\":[\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\",\\\"2019年财务处公共项目\\\"]}]\",\"CREATEDATETIME\":\"2019-04-26 10:48:21\",\"FIELD9\":\"budget_subFrom\",\"htmlUrl\":\"\",\"toWriteBudget\":\"\",\"orderStatus\":\"21\",\"budgetEndDate\":\"2019-12-31\",\"budgetStartDate\":\"2019-04-26\",\"processFlag\":\"\",\"number_budget\":\"B00000001\",\"CREATEUSER\":\"\",\"currency\":\"RMB\",\"orderCode\":\"\",\"budget_type\":\"F090002\",\"voucherCode\":\"\",\"DATAROWNUM\":1,\"budget_subFrom\":[{\"budget_superDepartement\":\" \",\"C_Type\":\"165\",\"budget_itemBalance\":\"20000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"20000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0.00\",\"DATAROWNUM\":5},{\"budget_superDepartement\":\" \",\"C_Type\":\"51\",\"budget_itemBalance\":\"872.03\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"5000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":6},{\"budget_superDepartement\":\" \",\"C_Type\":\"509\",\"budget_itemBalance\":\"150000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"150000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":7},{\"budget_superDepartement\":\" \",\"C_Type\":\"149\",\"budget_itemBalance\":\"10000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"10000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":8},{\"budget_superDepartement\":\" \",\"C_Type\":\"371\",\"budget_itemBalance\":\"8000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"8000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":9},{\"budget_superDepartement\":\" \",\"C_Type\":\"557\",\"budget_itemBalance\":\"50000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"50000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":10},{\"budget_superDepartement\":\" \",\"C_Type\":\"1344\",\"budget_itemBalance\":\"350000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"350000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":11},{\"budget_superDepartement\":\" \",\"C_Type\":\"1345\",\"budget_itemBalance\":\"50000.00\",\"budget_projectNum\":\"SDT-CWC-GGXM-2019\",\"budget_itemMoney\":\"50000.00\",\"budget_project\":\"SDT-CWC-GGXM-2019\",\"budget_itemDepartment\":\"01437000100004\",\"budget_superMoney\":\"0\",\"DATAROWNUM\":12}]}", Map.class);
//        List<Map<String, Object>> subs = (List<Map<String, Object>>) data.get("budget_subFrom");
//        new FormDataCodeToNameFilter().accept(data, subs, 1);
//        System.out.println(JSONTool.toJson(data));
//        System.out.println(JSONTool.toJson(subs));
//    }
}
