package com.changhong.epc.form.service.apply.impl;

import com.changhong.epc.bean.form.FormFieldEntity;
import com.changhong.epc.bean.form.apply.ApplyMerge;
import com.changhong.epc.bean.form.apply.ApplyMergeDto;
import com.changhong.epc.bean.form.apply.ApplyMergePageResult;
import com.changhong.epc.bean.report.Report;
import com.changhong.epc.config.tool.ContextEnvironment;
import com.changhong.epc.constter.form.apply.ApplyMergeMsg;
import com.changhong.epc.constter.system.SystemConst;
import com.changhong.epc.constter.system.businesscode.BudgetBusinessMeaningCode;
import com.changhong.epc.form.filter.data.FormDataFilterUtil;
import com.changhong.epc.form.mapper.tenant.apply.ApplyMergeDetailMapper;
import com.changhong.epc.form.mapper.tenant.apply.ApplyMergeMapper;
import com.changhong.epc.form.service.apply.ApplyMergeService;
import com.changhong.epc.form.service.apply.MergeService;
import com.changhong.epc.rely.api.epc.autoForm.FormApi;
import com.changhong.epc.rely.api.service.ReportApiService;
import com.github.pagehelper.PageInfo;
import com.iemsoft.framework.cloud.core.tools.Assert;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.core.tools.PageUtil;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class ApplyMergeServiceImpl implements ApplyMergeService, BudgetBusinessMeaningCode, ApplyMergeMsg {

    /**
     * 查询合并申请单列表显示的标题
     */
    public static final Set<String> TITLES = Collections.unmodifiableSet(
            new HashSet<>(
                    Arrays.asList(
                        // 申请单号
                        NUMBER_APPLY
                        // 申请日期
                        , BUDGET_START_DATE
                        // 申请人
                        , REIMBURSE_MAN
                        // 申请部门
                        , APPLY_DEPARTMENT
                        // 申请币种
                        , APPLYCATION_CURRENCY
                        // 申请金额
                        , APPLICATION_CURRENCY_AMOUNT
                        // 借款金额
                        , LOAN_AMOUNT
                    )
            )
    );

    @Resource
    private ApplyMergeMapper applyMergeMapper;

    @Resource
    private ApplyMergeDetailMapper applyMergeDetailMapper;

    @Resource
    private FormApi formApi;

    @Resource
    private ReportApiService reportApiService;

    @Resource
    private MergeService mergeService;

    @Override
    public ApplyMergePageResult getPageApplyMergeList(ApplyMergeDto applyMergeDto) {
        List<FormFieldEntity> fields =
        formApi.getFormFields(applyMergeDto.getFormId())
                .stream()
                .filter(field->Objects.equals(field.getParentsubFormNum(), 0))
                .filter(field->TITLES.contains(field.getFieldKey()))
                .map(field->new FormFieldEntity(field.getFieldKey(), field.getTitle()))
                .collect(Collectors.toList());

        Map<String, Object> applyInfo = applyMergeMapper.selectApplyInfo(applyMergeDto.getDataRowNum(), applyMergeDto.getFormId());
        if(ObjectUtil.empty(applyInfo)){
            return new ApplyMergePageResult(fields, new PageInfo<>());
        }
        PageUtil.setPage(applyInfo, applyMergeDto.getPageNum(), applyMergeDto.getPageSize());
        return new ApplyMergePageResult(fields, new PageInfo<>(
                FormDataFilterUtil.accept(applyMergeMapper.selectApplyMergeList(applyInfo, applyMergeDto.getFormId()))));
    }

    @Override
    public Report createReport(ApplyMergeDto applyMergeDto) {
        List<ApplyMerge> applyMerges = applyMergeMapper.select(new ApplyMerge(applyMergeDto.getDataRowNum()));
        // 合并过，直接返回
        if(ObjectUtil.notEmpty(applyMerges)){
            return getReport(applyMerges.stream().findFirst().map(ApplyMerge::getGroupId).orElse(""));
        }
        Assert.condition(applyMergeDto.isTryMerge(), APPLY_NOT_MERGE);
        // 合并申请单并返回groupId
        return getReport(mergeService.merge(applyMergeDto));
    }

    /**
     * 根据groupId生成用汇预算报表url
     * @param groupId
     * @return
     */
    public Report getReport(String groupId){
        ContextEnvironment context = new ContextEnvironment();
        return reportApiService.getReport(
                context.getSystemConfig(SystemConst.REMITTANCE_BUDGET_GROUP_KEY, SystemConst.REMITTANCE_BUDGET_REPORT)
                        .replace("{groupId}", groupId)
                , "");
    }

    @Override
    public void unMerge(ApplyMergeDto applyMergeDto) {

        // 验证是否有单据已经提交
        Assert.condition(applyMergeMapper.countOverApply(applyMergeDto) > 0, APPLY_FLOW_OVER);

        applyMergeMapper.deleteByDataRowNum(applyMergeDto.getDataRowNum());
        applyMergeDetailMapper.deleteByDataRowNum(applyMergeDto.getDataRowNum());
    }
}
