package com.changhong.epc.form.service.apply.impl;

import com.changhong.epc.bean.form.apply.ApplyMerge;
import com.changhong.epc.bean.form.apply.ApplyMergeDetail;
import com.changhong.epc.bean.form.apply.ApplyMergeDto;
import com.changhong.epc.bean.tenant.norm.extend.ApplyMergeSuppStander;
import com.changhong.epc.constter.count.format.ConstCountMoney;
import com.changhong.epc.constter.system.businesscode.BudgetBusinessMeaningCode;
import com.changhong.epc.form.filter.data.FormDataFilterUtil;
import com.changhong.epc.form.mapper.tenant.apply.ApplyMergeDetailMapper;
import com.changhong.epc.form.mapper.tenant.apply.ApplyMergeMapper;
import com.changhong.epc.form.service.apply.MergeService;
import com.changhong.epc.rely.api.epc.admin.CurrencyApi;
import com.changhong.epc.rely.api.tool.CurrencyTool;
import com.iemsoft.framework.cloud.core.tools.MsgTool;
import com.iemsoft.framework.cloud.core.tools.ObjectUtil;
import com.iemsoft.framework.cloud.core.tools.StringUtil;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;

@Service
public class MergeServiceImpl implements MergeService, BudgetBusinessMeaningCode, ConstCountMoney {

    public static final String CHILD_KEY    = "child";

    public static final String SUM_DAYS     = "sumDays";

    public static final String COUNTRIES    = "countries";

    @Resource
    private ApplyMergeMapper applyMergeMapper;

    @Resource
    private ApplyMergeDetailMapper applyMergeDetailMapper;

    @Resource
    private CurrencyApi currencyApi;

    @Override
    @Transactional
    public String merge(ApplyMergeDto applyMergeDto) {
        String groupId = StringUtil.getUUID();

        // 获取要合并的申请列表
        List<Map<String, Object>> applyInfos = getApplyInfos(applyMergeDto);
        // 验证是否可以合并
        validate(applyInfos);
        // 添加申请单合并信息
        addApplyMerges(applyInfos, applyMergeDto, groupId);
        return groupId;
    }

    /**
     * 验证是否可以合并
     * @param applyInfos
     */
    protected void validate(List<Map<String,Object>> applyInfos) {
        if(ObjectUtil.empty(applyInfos)){
            return;
        }
        // 部门相同

        // 出差日期相同

        // 申请币种相同

    }

    /**
     * 添加申请单合并信息
     * @param applyInfos
     */
    protected void addApplyMerges(List<Map<String,Object>> applyInfos, ApplyMergeDto applyMergeDto, String groupId) {
        List<ApplyMergeSuppStander> suppStanders = applyMergeDetailMapper.selectSuppStander(applyMergeDto.getFormId());
        // 添加其他费用
        ApplyMergeSuppStander otherMoney = new ApplyMergeSuppStander();
        otherMoney.setType(OTHER_COST);
        otherMoney.setTypeName(MsgTool.get(OTHER_COST));
        suppStanders.add(otherMoney);

        // 记录国家信息
        Map<String, String> countries = new LinkedHashMap<>();

        // 申请单合并信息
        List<ApplyMerge> applyMerges = new ArrayList<>();

        applyInfos.stream().forEach(apply->{
            applyMerges.add(addApplyMerge(apply, countries, groupId));
            // 计算申请单报销项目明细信息
            countApplyMergeDetail((List<Map<String,Object>>) apply.get(CHILD_KEY), suppStanders);
        });

        applyMerges.stream()
                .map(apply->{
                    apply.setApplyCountries(String.join(",", countries.keySet()));
                    apply.setApplyCountriesName(String.join(",", new LinkedHashSet<>(countries.values())));
                    return apply;
                })
                .forEach(applyMergeMapper::insertSelective);

        // 获取申请币种对本位币汇率
//        String exchangeRate = applyInfos.stream().findFirst().map(info->info.get(EXCHANGE_RATE)).map(Object::toString).orElse("");
        String applyCurrency    = applyInfos.stream().findFirst().map(info->info.get(APPLYCATION_CURRENCY)).map(Object::toString).orElse("");
        String applyDate        = applyInfos.stream().findFirst().map(info->info.get(BUDGET_START_DATE)).map(Object::toString).orElse("");
        String exchangeRate     = currencyApi.exchangeRate(applyCurrency, CurrencyTool.USD, applyDate);

        // 添加申请单详细
        suppStanders.stream()
                .filter(supp->supp.getMoney() != null && !Objects.equals("0", supp.getMoney()))
                .map(supp->{
                    ApplyMergeDetail applyMergeDetail = new ApplyMergeDetail();
                    applyMergeDetail.setGroupId(groupId);
                    applyMergeDetail.setExpenseType(supp.getType());
                    applyMergeDetail.setExpenseTypeName(supp.getTypeName());
                    applyMergeDetail.setExpenseWay(supp.getName());
                    applyMergeDetail.setExpenseWayName(MsgTool.get(supp.getName()));
                    applyMergeDetail.setCostMoney(supp.getMoney());
                    applyMergeDetail.setCostMoneyUsd(CurrencyTool.valueOf(exchangeRate, supp.getMoney()));
                    applyMergeDetail.initParam();
                    return applyMergeDetail;
                })
                .forEach(applyMergeDetailMapper::insertSelective);
    }

    protected ApplyMerge addApplyMerge(Map<String, Object> applyInfo, Map<String, String> countries, String groupId){
        Map<String, Object> applyInfoName = FormDataFilterUtil.accept(new HashMap<>(applyInfo));
        ApplyMerge applyMerge = new ApplyMerge();
        applyMerge.setGroupId(groupId);

        String[] countrieCodes = Optional.ofNullable(applyInfo.get(COUNTRIES)).map(Object::toString).orElse("").split(",");
        String[] countrieNames = Optional.ofNullable(applyInfoName.get(COUNTRIES)).map(Object::toString).orElse("").split(",");

        for (int i = 0, j = Math.min(countrieCodes.length, countrieNames.length); i < j; i++) {
            countries.put(countrieCodes[i], countrieNames[i]);
        }

        applyMerge.setApplyId(NumberUtils.createInteger(Objects.toString(applyInfo.get(DATAROWNUMBER), "0")));
        applyMerge.setApplyNumber(getStringVal(applyInfo, NUMBER_APPLY));
        applyMerge.setApplyMoney(getStringVal(applyInfo, APPLICATION_CURRENCY_AMOUNT));
        applyMerge.setApplySumDays(NumberUtils.createDouble(Objects.toString(applyInfo.get(SUM_DAYS), "0")).intValue());
        applyMerge.setApplyDate(getStringVal(applyInfo, BUDGET_START_DATE));
        applyMerge.setApplyLoanAmount(getStringVal(applyInfo, LOAN_AMOUNT));

        // 币种
        applyMerge.setApplyCurrency(getStringVal(applyInfo, APPLYCATION_CURRENCY));
        applyMerge.setApplyCurrencyName(getStringVal(applyInfoName, APPLYCATION_CURRENCY));
        // 申请人
        applyMerge.setApplyPerson(getStringVal(applyInfo, REIMBURSE_MAN));
        applyMerge.setApplyPersonName(getStringVal(applyInfoName, REIMBURSE_MAN));
        // 申请部门
        applyMerge.setApplyDepartment(getStringVal(applyInfo, APPLY_DEPARTMENT));
        applyMerge.setApplyDepartmentName(getStringVal(applyInfoName, APPLY_DEPARTMENT));

        applyMerge.initParam();
        return applyMerge;
    }

    protected String getStringVal(Map<String, Object> info, String key){
        return Optional.ofNullable(info).map(i->i.get(key)).orElse("").toString();
    }

    /**
     * 添加申请单报销项目明细信息
     * @param applyChildInfos
     */
    protected void countApplyMergeDetail(List<Map<String,Object>> applyChildInfos, List<ApplyMergeSuppStander> suppStanders) {
        if(ObjectUtil.empty(applyChildInfos)){
            return;
        }
        applyChildInfos.stream().forEach(child->{
            suppStanders.stream().forEach(supp->{
                supp.appendMoney(
                        // 添加总额
                        Optional.ofNullable(child.get(supp.getType()+"_count"))
                                    .map(Object::toString)
                                .orElse(
                                        // 添加原始金额
                                        Optional.ofNullable(child.get(supp.getType()))
                                                .map(Object::toString)
                                        .orElse("0")
                                )
                );
            });
        });
    }

    protected List<Map<String, Object>> getApplyInfos(ApplyMergeDto applyMergeDto){
        // 查询需要合并的申请单信息
        Set<Integer> applyIds = new HashSet<>(
                Optional.ofNullable(applyMergeDto).map(ApplyMergeDto::getJoinDataRowNum).orElse(Collections.EMPTY_LIST));
        applyIds.add(applyMergeDto.getDataRowNum());
        List<Map<String, Object>> applyInfos = applyMergeMapper.selectApplyMergeListByIds(applyIds, applyMergeDto.getFormId());
        if(ObjectUtil.empty(applyIds)){
            return Collections.EMPTY_LIST;
        }
        // 查询申请单子表单单信息
        List<Map<String, Object>> applyChilds = applyMergeMapper.selectApplyMergeChildList(applyIds, applyMergeDto.getFormId());
        if(ObjectUtil.empty(applyChilds)){
            return applyInfos;
        }
        Iterator<Map<String, Object>> applyIte = applyInfos.iterator();
        Map<String, Object> apply = applyIte.next();
        for(Map<String, Object> applyChild : applyChilds){
            if(!Objects.equals(applyChild.get(SUNMDATAROWNUMBER), apply.get(DATAROWNUMBER))){
                if(applyIte.hasNext()) {
                    apply = applyIte.next();
                }else{
                    apply = null;
                }
            }
            addChild(apply, applyChild);
        }

        return applyInfos;
    }

    protected void addChild(Map<String, Object> apply, Map<String, Object> applyChild){
        if(ObjectUtil.empty(apply)){
            return;
        }
        List<Map<String, Object>> child = (List<Map<String, Object>>) apply.get(CHILD_KEY);
        if(child == null){
            apply.put(CHILD_KEY, child = new ArrayList<>());
        }
        child.add(applyChild);
    }
}
