package com.changhong.epc.form.service.data.impl;

import com.changhong.autoform.entity.FormData;
import com.changhong.autoform.entity.SelectForm;
import com.changhong.autoform.entity.page.PageResult;
import com.changhong.autoform.entity.sql.Where;
import com.changhong.autoform.entity.sql.Where.Conditions;
import com.changhong.autoform.entity.sql.Where.JoinType;
import com.changhong.autoform.entity.sql.Where.Type;
import com.changhong.autoform.entity.sql.select.Select;
import com.changhong.autoform.mapper.data.impl.DataMapperImpl;
import com.changhong.epc.bean.form.FormBaseEntity;
import com.changhong.epc.bean.form.FormKey;
import com.changhong.epc.bean.parsing.BillInfo;
import com.changhong.epc.constter.base.BaseConst;
import com.changhong.epc.constter.base.Context;
import com.changhong.epc.constter.base.MQConst;
import com.changhong.epc.constter.form.FormConst;
import com.changhong.epc.constter.system.SystemClients;
import com.changhong.epc.constter.system.businesscode.BudgetBusinessMeaningCode;
import com.changhong.epc.form.filter.data.FormDataFilterUtil;
import com.changhong.epc.form.mapper.tenant.FormDataMapper;
import com.changhong.epc.form.mapper.tenant.bill.BillInfoMapper;
import com.changhong.epc.form.service.field.FormFieldService;
import com.iemsoft.framework.cloud.config.datasource.DynamicDataSource;
import com.iemsoft.framework.cloud.core.exception.IEMRuntimeException;
import com.iemsoft.framework.cloud.core.thread.ThreadData;
import com.iemsoft.framework.cloud.core.tools.JSONTool;
import com.iemsoft.framework.cloud.core.tools.SpringUtil;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.ui.ModelMap;

import javax.annotation.Resource;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

@Component("dataExtendImpl")
@Slf4j
public class DataExtendImpl extends DataMapperImpl implements SystemClients, BudgetBusinessMeaningCode, MQConst, FormConst {

	@Resource(name="formFieldService")
	private
	FormFieldService formFieldService;

	@Resource
	private BillInfoMapper billMapper;

	@Resource
	private FormDataMapper formDataMapper;

	@Resource
	private RedisTemplate redisTemplate;
	
	@Resource
	private com.changhong.epc.form.service.field.FormFieldService FormFieldService;

	@Override
	protected Connection getConnection() {
//		MybatisTool.initDataSource();
		try {
			return SpringUtil.getBean(DynamicDataSource.class).getConnection();
		} catch (SQLException e) {
			log.error(e.getMessage(), e);
			throw new IEMRuntimeException(e.getMessage());
		}
	}

	@Transactional
	@Override
	public  int insertData(FormData formData) {
		log.debug("缺失数据问题-添加数据:{}", JSONTool.toJson(formData));
		//首先验证数据唯一
		verifier(formData);
		//如果有票据信息添加到billInfo表中
		//修改info表表单数据
		int id = super.insertData(formData);
		formData.setDataRowNum(id);
		if(this.getOrderCode(formData) != FormStateType.falze){
			Map<String,Object> maps = formData.getData();
			maps.put("orderStatus", AUTO_FORM_ING);
			formDataMapper.updataFormByCustom(formData.getFormName(), id, maps);
		}

		if(formData.isAuto()){
			Map<String, Object> param = new ModelMap("tenantID" ,formData.getTenantID())
					.addAttribute("formID", formData.getFormID())
					.addAttribute("id", id)
					.addAttribute("field", formData.getAutoField())
					.addAttribute("fieldKey", formData.getAutoFieldKey())
					.addAttribute("fieldSize", formData.getAutoFieldSize());
			log.debug("发送{}消息,参数:{}", AUTO_NUMBER, param);
			redisTemplate.convertAndSend(AUTO_NUMBER
					 , Context.getMQParam(param)
			);
//			String autoNumber = FormFieldService.getAutoNumber(new ModelMap("tenantID" ,formData.getTenantID())
//					 							.addAttribute("formID", formData.getFormID())
//					 							.addAttribute("field", formData.getAutoField())
//					 							.addAttribute("id", id));
//			maps.put(formData.getAutoField(),autoNumber);
		}
//		 if(ObjectUtil.notEmpty(maps)) {
//		 }
		 
		 return id;
	}
	
	@Transactional
	@Override
	public int updateDate(FormData formData){
		log.debug("缺失数据问题-修改数据:{}", JSONTool.toJson(formData));
		//修改验证唯一
		//首先验证数据唯一
		verifier(formData);
		int result = super.updateDate(formData);
		Map<String,Object> maps = new HashMap<>();
		if(this.getOrderCode(formData).equals(FormStateType.insert)){
			maps.put("orderStatus", AUTO_FORM_ING);
			formDataMapper.updataFormByCustom(formData.getFormName(), formData.getDataRowNum(), maps);
		}
		return result;
	}
	
	
	/**
	 * 取出dataruownum
	 */
	public int getRow(FormData formData){
		Object count = selectOne(new Select(formData.getFormName()
				, new Where(new Conditions("tenantID",Type.EQ,formData.getTenantID())
						, new Conditions("deleteFlg", Type.EQ, JoinType.AND, 0))
				, new String[]{COUNT_KEY})).get(COUNT_KEY);
		
		return Integer.parseInt(count.toString())+1;
	}
	

	public FormStateType getOrderCode(FormData formData){
		if(formData.getData().get("orderCode") != null && !"".equals(Objects.toString(formData.getData().get("orderCode"),"")) ){
			BillInfo bill = new BillInfo();
			String[] formId = formData.getFormName().split("_");
			//表示票据已经上传,但是没有票据信息返回 状态为666
			bill.setDataStart(AUTO_FORM_ING);
			bill.setFormId(formId[formId.length-1]);
			bill.setFormDataId(formData.getDataRowNum());
			bill.setCtId(Objects.toString(formData.getData().get(COST_TYPE)," "));
			bill.initParam();
			bill.setOrderCode(formData.getData().get("orderCode").toString());
			int result = billMapper.updateBill(bill);
			// System.out.println("添加票据返回结果："+result);
			if(result >0)
				return FormStateType.update;
			
			if(result == 0){
				return billMapper.insert(bill) >0 ? FormStateType.insert :FormStateType.falze;
			}
		}
		return FormStateType.falze;
	}
	@Transactional
	public int deleteDate(FormData formData) {
		return super.deleteDate(formData);
	}
	

	/**
	 * 验证唯一字段
	 */
	public void verifier(FormData formData){
//		log.debug("开始表单数据唯一验证--->");
//		log.debug("当前表单数据:"+ JSONTool.toJson(formData.getData()));
//		log.debug("当前表单:"+ formData.getFormName());
//		Integer dataRowNum = formData.getDataRowNum();
		//获取表单控件属性
//		Map<String,OnlyValue> fields = getfields(formData.getFormID(),formData.getTenantID());
//		//检查字段是否唯一
//		fields.keySet().stream().forEach(o->{
//			OnlyValue on = fields.get(o);
//			Integer i = formDataMapper.selectValueCount(formData.getFormName(),o,Objects.toString(formData.getData().get(o),null),dataRowNum);
//			Assert.condition(i>0, "C00021",Objects.toString(on.getFieldName(),""));
//		});
		
	}

	//修改表单后回写行程id
	@Override
	public int addSubForm(Map<String, Object> map,FormData formData,String formName){
		log.debug("当前表单数据:"+ JSONTool.toJson(formData));
		//子表单id
		String dataN =  Objects.toString(map.get(DATAROWNUMBER));
		map.remove(DATAROWNUMBER);
		int id = super.addSubForm(map,formData,formName);
		Map<String, Object> m = formData.getData();
		String orderCode = Objects.toString(m.get(ORDER_CODE),null);
		//票据回写
		if(this.eqJourney(formName,orderCode)){
			log.debug("行程id回写------------------>>>");
			log.debug("回写单号为:"+orderCode);
			billMapper.updateJourney(dataN,id,orderCode);
		}
		return id;
	}
	//是否有行程
	public boolean eqJourney(String formName,String orderCode){
		String [] s = formName.split("_");
		return (orderCode!=null && !"".equals(orderCode)) && JOURNEY.equals(s[s.length-1]);
	}
//	public static void main(String...a){
//		String [] s = "form333ju".split("_");
//		// System.out.println(s[s.length-1]);
//	}

	public Map<String,OnlyValue> getfields(String formId,String tenantId){
		FormKey fk = new FormKey();
		fk.setFormId(formId);
		fk.setTenantId(tenantId);
		FormBaseEntity fb=  formFieldService.selectFormFieldProperty(fk);
		if(fb == null || Objects.equals("",fb.getFieldset()) || fb.getFieldset() == null){
			return new HashMap<>();
		}
		Map<String,OnlyValue> fiels = new HashMap<>();
		Map<String,Object> data = JSONTool.toObj(fb.getFieldset(),Map.class);
		data.keySet().stream().forEach(k->{
			Map<String,Object> map1 = JSONTool.toObj(data.get(k).toString(),Map.class);
			if(Objects.equals(Objects.toString(map1.get(TAG_TYPE),null),TEXT)
					&& Objects.equals(Objects.toString(map1.get(PARENTSUBFORMNUM),null),"0")) {
				Map<String, Object> map2 = JSONTool.toObj(JSONTool.toJson(map1.get(TAG_ATTRIBUTE)), Map.class);
				Boolean flag = Boolean.parseBoolean(Objects.toString(map2.get(ONLY_VERIFICATION),"false"));
				if(map2.get(ONLY_VERIFICATION)!=null
						&& flag){
					fiels.put(Objects.toString(map2.get(ALIAS),null),new OnlyValue(map2.get(TITLE),flag));
				}
			}
		});
		return fiels;
	}

	@Override
	public Map<String, Object> selectOne(SelectForm selectForm) {
		ThreadData.set(BaseConst.IGNORE_DATA_POWER, true);
		Map<String, Object> res = super.selectOne(selectForm);
		log.debug("缺失数据问题-查询数据:{}, 返回:{}", JSONTool.toJson(selectForm), JSONTool.toJson(res));
		return res;
	}

	@Override
	public PageResult<Map<String, Object>> selectPage(SelectForm selectForm) {
		ThreadData.set(BaseConst.IGNORE_DATA_POWER, !selectForm.isDataPower());
		PageResult<Map<String, Object>> pages = super.selectPage(selectForm);
		FormDataFilterUtil.accept(pages.getList());
		log.debug("缺失数据问题-列表数据:{}, 返回:{}", JSONTool.toJson(selectForm), JSONTool.toJson(pages));
		return pages;
	}
}
@Data
@AllArgsConstructor
@NoArgsConstructor
class OnlyValue{
	private Object fieldName;
	private Boolean onlyVerification;
}

enum FormStateType{
	insert,
	update,
	falze
}
